
# q-expansion of newform 90.6.a.d, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 90
weight = 6
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[4], [0], [-25], [-172], [-132], [-946], [222], [500], [-3564], [-2190], [2312], [-11242], [-1242], [20624], [-6588], [21066], [-7980], [16622], [1808], [24528], [20474], [-46240], [51576], [110310], [-78382], [-141942], [-436], [-232968], [-174850], [-182994], [-122452], [241908], [-277098], [-193540], [-140550], [433952], [-555922], [-66616], [205692], [-433854], [489180], [719462], [185928], [-591406], [-449478], [157160], [253052], [1073444], [626832], [-116650], [743046], [-978720], [-1132798], [-905652], [-1939938], [805476], [858690], [-383608], [2010758], [-202602], [-221536], [322506], [1443008], [-171312], [-1026886], [-752958], [1994132], [-987022], [-2206008], [2741870], [2389566], [279480], [-2476372], [2745254], [-1189060], [-3257604], [-1983510], [4974158], [1342638], [-1094230], [954060], [-1593898], [2646648], [3723554], [-2583400], [-7562064], [-4307730], [-2243542], [-1656702], [-2891596], [6526992], [5962320], [2991908], [1204188], [9205460], [3359556], [2537010], [9315798], [-5028016], [134222], [605648], [7062402], [10302936], [-1047690], [14076452], [1626818], [-6960888], [11363886], [-14868840], [-1236118], [-12449812], [-8734906], [-12549498], [-14665780], [-196288], [11159598], [-2251576], [-8053188], [416466], [-13172100], [-1694938], [-8916046], [14289402], [5333136], [698252], [-17987982], [-13946410], [-6222720], [-7767292], [24208334], [12684980], [-19763244], [-9017608], [-1125562], [-22574682], [-632350], [12586746], [21579248], [30976002], [-4249290], [3423332], [-31057062], [-31190356], [8284872], [-18168850], [10274280], [6285974], [-15405018], [14352620], [-13327764], [32484638], [-15460002], [-1694776], [2872572], [39542168], [-11317752], [8513480], [7545870], [-18450022], [-6750462], [-6456768], [39664806], [-34301452], [5771148], [-7087458], [-45936204], [-45029848], [-23736442]]
